﻿using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;
using System.Collections;
using System.Diagnostics;
using System.Linq;

namespace SimpleGrid
{
	public partial class SimpleGridViewController : UIViewController
	{
		#region Members

		IGGridView gridView;

		customHelper dsh;

		List<cSimpleGrid> lstGridItems;


        #endregion

        #region Constructor

        public SimpleGridViewController () : base ("SimpleGridViewController", null)
		{
		}

		#endregion

		#region View Controller Methods

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

	    public override void DidRotate(UIInterfaceOrientation fromInterfaceOrientation)
	    {
            gridView.ReloadData();

        }

	    public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();


            if (UIDevice.CurrentDevice.CheckSystemVersion(7, 0))
            {

                this.EdgesForExtendedLayout = UIRectEdge.None;
                this.AutomaticallyAdjustsScrollViewInsets = false;
            }


            //Next, create a new instance of the grid view and place it on the view.
            gridView = new IGGridView (new System.Drawing.RectangleF (0, 0, this.View.Bounds.Width, this.View.Bounds.Height), IGGridViewStyle.IGGridViewStyleDefault);
			gridView.AutoresizingMask = UIViewAutoresizing.All;
		    gridView.HeaderHeight = 0;
            gridView.SelectionType = IGGridViewSelectionType.IGGridViewSelectionTypeNone;
	        gridView.RowSeparatorColor = UIColor.Black;
	        gridView.RowSeparatorHeight = 1;
	        gridView.ColumnSpacing = 1;
	        gridView.BackgroundColor = UIColor.LightGray;
			this.View.Add (gridView);

         

		    //Setup the Grid
			SetupGrid ();
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is responsible for the layour and populate of the grid view
		/// </summary>
		private bool SetupGrid ()
		{
			//First, create a new return variable
			bool bReturn = true;

			try {

				// We're using the basic IGGridViewDataSourceHelper
				// since our data will be displayed in a standard datagrid
				dsh = new customHelper();

				// Since we are manually creating columns, turn off auto-generation
				dsh.AutoGenerateColumns = false;


                customGoalsColumnDefinition type1 = new customGoalsColumnDefinition("PT", this);
                dsh.ColumnDefinitions.Add(type1);
             


                dsh.AutoGenerateColumns = true;

                dsh.AllowEditing = true;

				//Set the DataSource
				gridView.DataSource = dsh;

				//Turn off empty rows
				gridView.EmptyRows = false;

				//Update gridView
				gridView.UpdateData ();

			} catch {
				//Error building the grid, return false.
				bReturn = false;
			}

			//Finally, return the return variable
			return bReturn;
		}

		#endregion

		public class customGoalsColumnDefinition : IGGridViewColumnDefinition
		{
			#region Members

			string _fieldKey;
			SimpleGridViewController mController;

			#endregion

			#region Constructor

			public customGoalsColumnDefinition (string key, SimpleGridViewController controller) : base ()
			{
				_fieldKey = key;
				mController = controller;

			}

			#endregion

			#region Properties

			public override string FieldKey {
				get {
					return _fieldKey;
				}
			}

		    public override bool Editable 
		    {
				get {
					return true;
				}
			}

			#endregion

			#region Override Methods


        
            

			public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
			{
                IGGridViewCell cell = (IGGridViewCell)gridView.DequeueReusableCell("Cell");

                if (cell == null)
                {
                    cell = new IGGridViewCell("Cell");
                }

                cell.TextLabel.Text = $"{path.RowIndex}.{path.ColumnIndex}";

                return cell;
			}


		    public override void DisplayEditor(IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
		    {


                dataSource.EditingEndTransitionOptions = UIViewAnimationOptions.TransitionNone;
                dataSource.EditingStartTransitionOptions = UIViewAnimationOptions.TransitionNone;
                dataSource.EditingTransitionDuration =0;

                base.DisplayEditor(gridView, path, dataSource);
		    }

		    public override void RemoveEditor (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource, bool cellVisible, bool anotherCellEnteringEditMode)
			{
				gridView.UnregisterKeyboard (!anotherCellEnteringEditMode);

				IGGridViewCell cell = (IGGridViewCell)gridView.ResolveCellForPath (path);

		        if (cell.Subviews.Length > 0 && cell.Subviews[0] is UITextField)
		        {
		            UITextField field = (UITextField) cell.Subviews[0];
		            cell.TextLabel.Text = field.Text;

				    UIView.Transition (field, cell.TextLabel, 0, UIViewAnimationOptions.TransitionNone, null);
                }

                //Before returning the cell, apply the styling to the cell
                //ApplyStylingToCell(cell, path.RowIndex, gridView);

				//THIS IS THE LINE OF CODE THAT I USE WHEN A USER POPULATES THE BLANK ROW, TO AUTO RE-ADD A NEW BLANK ROW AT THE END, SO THERE IS ALWAYS A BLANK ROW.
				//gridView.UpdateData ();

			}

			#endregion

		}

		public class customHelper : IGGridViewDataSourceHelper
		{
            private IGGridViewCell _editCell;

         


            public override IGGridViewCell CreateCell(IGGridView gridView, IGCellPath path)
            {


 
                var colDef = this.ColumnDefinitions.GetItem<NSObject>(0) as IGGridViewColumnDefinition;

                if (colDef != null)
                {
                    IGGridViewCell cell = colDef.CreateCell(gridView, path, this);

                    if (this.AllowEditing && colDef.Editable)
                    {
                        var tapGesture = new UITapGestureRecognizer(this, new MonoTouch.ObjCRuntime.Selector("HandleEdit:"))
                        {
                            NumberOfTapsRequired = 1
                        };

                        cell.RegisterTemporaryGestures(new NSObject[] { tapGesture });

                    }


                    return cell;
                }

                return null;
            }

            [Export("HandleEdit:")]
            private void HandleEdit(UITapGestureRecognizer gestureRecognizer)
            {

                gestureRecognizer.Reset();

                var cell = gestureRecognizer.View as IGGridViewCell;
                if (cell == null)
                    return;

                if (_editCell != null && cell != _editCell)
                {
        
                    var lastCol = this.ColumnDefinitions.GetItem<NSObject>(0) as IGGridViewColumnDefinition;
                    lastCol.RemoveEditor(_editCell.GridView, _editCell.Path, this, true, true);
                }

                _editCell = cell;


 
                var colDef = this.ColumnDefinitions.GetItem<NSObject>(0) as IGGridViewColumnDefinition;
                colDef.DisplayEditor(cell.GridView, cell.Path, this);

    

            }



            public override void DidEndDisplayingCell(IGGridView gridView, IGGridViewCell cell, IGCellPath path)
            {
                if (cell == _editCell)
                {
                    this.EndEditing();
                    _editCell = null;

                }
            }

            public override void EndEditing()
            {
                if (_editCell != null)
                {
  
                    var colDef = this.ColumnDefinitions.GetItem<NSObject>(0) as IGGridViewColumnDefinition;
                    colDef.RemoveEditor(_editCell.GridView, _editCell.Path, this, false, false);
                }
            }

            public override int NumberOfRowsInSection(IGGridView gridView, int section)
            {
                return (int)15;
            }


		    public override IGGridViewCell CreateFixedLeftCell(IGGridView gridView, IGCellPath path)
		    {
		        var cell = (IGGridViewCell)gridView.DequeueReusableCell("leftcell") ??
		                   new IGGridViewCell("leftcell");

                cell.BackgroundColor = UIColor.Blue;
		        cell.TextLabel.Text = $"{path.RowIndex}";
		        cell.TextLabel.TextColor = UIColor.White;
		        return cell;
		    }


		    public override IGColumnWidth GetWidthForFixedLeftColumn(IGGridView gridView, int column)
		    {
                return IGColumnWidth.CreateNumericColumnWidth(100);
            }

		    public override int NumberOfFixedLeftColumns(IGGridView gridView)
		    {
		        return 1;
		    }

		    public override int NumberOfColumns(IGGridView gridView)
            {
                return 523;
            }

            public override bool ProvideRowHeights(IGGridView gridView)
            {
                return true;
            }

            public override float ResolveRowHeight(IGGridView gridView, IGRowPath path)
            {

                if (path.RowIndex < 3 || path.RowIndex >= 12)
                    return 30;

                var h = (gridView.Bounds.Height - 6*30  )/ 9;
                return h;
            }

            public override IGColumnWidth ResolveColumnWidth(IGGridView gridView, int nint)
            {
                var currOrient = UIDevice.CurrentDevice.Orientation;
                return (currOrient.IsPortrait()) ? IGColumnWidth.CreateNumericColumnWidth(100) : IGColumnWidth.CreateNumericColumnWidth(150);

            }
            public override int NumberOfSections(IGGridView gridView)
            {
                return 1;
            }



        }

	
	}

  
}

